#include <iostream>
#include <cstdio>
#include <map>
#include <iomanip>
#include <vector>
#include <algorithm>
#include <queue>
using namespace std;

const int MAXN = 100010;

class SegmentTreeWithLazy{
    int n;
    long long int tree[MAXN*4];
    long long int lazy[MAXN*4];
    long long int arr[MAXN];

    public:
    SegmentTreeWithLazy(vector<int> &arr){
        for(int i=0; i<arr.size(); i++){
            this->arr[i] = arr[i];
        }
        n = arr.size();
        build(1, 0, n-1);
    }

    void build(int node, int start, int end){
        lazy[node] = 0;
        if (start == end){
            tree[node] = arr[start];
        }
        else{
            int mid = (start + end) / 2;
            build(2*node, start, mid);
            build(2*node+1, mid+1, end);
            tree[node] = min(tree[2*node], tree[2*node+1]);
        }
    }

    void update(int node, int start, int end, int l, int r, int val){
        if (lazy[node] != 0){
            tree[node] += lazy[node];
            if (start != end){
                lazy[2*node] += lazy[node];
                lazy[2*node+1] += lazy[node];
            }
            lazy[node] = 0;
        }
        if (start > end || start > r || end < l){
            return;
        }
        if (start >= l && end <= r){
            tree[node] += val;
            if (start != end){
                lazy[2*node] += val;
                lazy[2*node+1] += val;
            }
            return;
        }
        int mid = (start + end) / 2;
        update(2*node, start, mid, l, r, val);
        update(2*node+1, mid+1, end, l, r, val);
        tree[node] = min(tree[2*node], tree[2*node+1]);
    }

    long long int query(int node, int start, int end, int l, int r){
        if (start > end || start > r || end < l){
            return 1000000000;
        }
        if (lazy[node] != 0){
            tree[node] += lazy[node];
            if (start != end){
                lazy[2*node] += lazy[node];
                lazy[2*node+1] += lazy[node];
            }
            lazy[node] = 0;
        }
        if (start >= l && end <= r){
            return tree[node];
        }
        int mid = (start + end) / 2;
        int p1 = query(2*node, start, mid, l, r);
        int p2 = query(2*node+1, mid+1, end, l, r);
        return min(p1, p2);
    }

    void print_list(){
        for (int i=0; i<n; i++){
            cout << query(1, 0, n-1, i, i) << " ";
        }
        cout << endl;
    }
};

class Edge{
    public:
    int a, b;
    int c;
    Edge(): a(0), b(0), c(0) {}
    Edge(int a, int b, int c): a(a), b(b), c(c) {}
};

class DisjointSetUnion{
    public:
    int parent[MAXN];
    int N;
    DisjointSetUnion(int _N){
        N = _N;
        for (int i=0; i<N; i++){
            parent[i] = i;
        }
    }
    int find(int a){
        return parent[a] == a ? a : parent[a] = find(parent[a]);
    }
    void merge(int a, int b){
        parent[find(a)] = find(b);
    }
};

bool banned[MAXN];
vector<pair<int,int> > tree[MAXN];
int mft[MAXN];

void dfs(int node, int parent, int minf){
    mft[node] = minf;
    for (auto child : tree[node]){
        if (child.first != parent){
            dfs(child.first, node, min(minf, child.second));
        }
    }
}


int N, R;
int process[MAXN];

// instead of dfs use bfs write the function below
void bfs(){
    int x = 0;
    for(int i=0; i<N; i++){
        mft[i] = -1;
    }
    queue<int> q;
    q.push(0);
    mft[0] = 100000000;

    while(!q.empty()){
        x++;
        // if(x<= 20){
        //     cout << x << " " << q.front() << endl;
        // }
        int node = q.front();
        q.pop();
        for(auto child: tree[node]){
            if(mft[child.first] == -1){
                mft[child.first] = min(mft[node], child.second);
                q.push(child.first);
                // if(x <= 20){
                //     cout << "ADD: " << node << " -> " << child.first << " " << child.second << endl;
                // }
            }
        }
    }
}

int main(){
    // int S;
    // int T;
    // cin >> S >> T;
    int T = 1;


    vector<Edge> edges;
    vector< pair<int,int> > vertices;

    for (int t = 0; t < T; t++){
        // file name should be input_t.in
        // FILE *f = fopen(("input_" + to_string(S+t) + ".in").c_str(), "w");
        // cout << t << std::endl;

        int a, b, c;

        cin >> N >> R;
        // fprintf(f, "%d %d\n", N, R);
        // cout << N << "  "<< R << std::endl;

        for(int i=0; i<N; i++){
            tree[i].clear();
        }
        edges.resize(R);
        vertices.resize(N);
        // cout << 1 << endl;

        for (int i=0; i<N;i++){
            cin >> process[i];
            // fprintf(f, "%d ", process[i]);
            vertices[i] = make_pair(process[i], i);
        }
        sort(vertices.begin(), vertices.end());

        // cout << 2 << endl;

        for (int i=0; i<R; i++){
            cin >> a >> b >> c;
            // fprintf(f, "%d %d %d\n", a, b, c);
            a--;
            b--;
            edges[i] = Edge(a,b,c);
        }
        // fclose(f);

        // cout << 3 << endl;

        sort(edges.begin(), edges.end(), [](const Edge& e1, const Edge& e2) {
            return e1.c > e2.c;
        });

        DisjointSetUnion dsu(N);
        for(auto e : edges){
            if (dsu.find(e.a) != dsu.find(e.b)){
                dsu.merge(e.a, e.b);
                tree[e.a].push_back(make_pair(e.b, e.c));
                tree[e.b].push_back(make_pair(e.a, e.c));
            }
        }
        // cout << 4 << endl;

        for(int i=0; i<N; i++){
            mft[i]=-1;
        }
        // cout << 42 << endl;

        // dfs(0, -1, 100000000);
        bfs();
        // for(int i=0; i<N; i++){
        //     cout << mft[i] << " ";
        // }
        // cout << endl;

        // cout << 5 << endl;

        vector< pair<int,int> > flood_pairs;
        flood_pairs.resize(N);
        for (int i=0; i<N; i++){
            if (mft[i] == -1){
                mft[i] = 0;
                banned[i] = true;
                // cout << "BAN: " << i << std::endl;
            }
            else{
                banned[i] = false;
            }
            flood_pairs[i] = make_pair(mft[i], i);
        }
        // cout << 6 << endl;

        sort(flood_pairs.begin(), flood_pairs.end());

        // cout << 7 << endl;
        vector<int> flood_times;
        flood_times.resize(N);

        int pos[MAXN];
        for (int i=0; i<N; i++){
            flood_times[i] = flood_pairs[i].first;
            pos[flood_pairs[i].second] = i;
        }
        // cout << 8 << endl;

        SegmentTreeWithLazy st(flood_times);

        int result = 0;

        // st.print_list();
        // cout << 9 << endl;
        int x = 0;
        for (const auto vertex:vertices){
            // x ++;
            // if(x <= 10){
            //     cout << vertex.second  << "   " << mft[vertex.second]  << endl;
            // } 
            

            if (banned[vertex.second]){
                continue;
            }
            int index = pos[vertex.second];
            st.update(1, 0, N-1, index, N-1, -vertex.first);
            if(st.query(1, 0, N-1, 0, N-1) >= 0 ){
                result ++;
                // cout << st.query(1, 0, N-1, 0, N-1) << endl;
                // cout << "ADD: " << vertex.second << std::endl;
                // st.print_list();
            }
            else{
                st.update(1, 0, N-1, index, N-1, vertex.first);
            }
        }
        
        cout << result << endl;
        // cout << "Data Set " << t+1 << ":" << endl << result << endl << endl;
    }

    return 0;
}